!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! Program for reading the PALM TIME-SERIES OUTPUT NETCDF Data      !!!
!!! This Code must be compiled with the following flags/options:     
!!! gfortran -o read_palm_3d.x -I/usr/include -L/usr/lib64 -lnetcdf -lnetcdff /usr/lib64/libnetcdf.a read_palm_3d.f90
!!! Last Updated (February 19, 2021)                                 !!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	Program read_palm_3d 

	USE netcdf
	implicit none

	Character(len=*), Parameter :: FILE_NAME = "TEST.nc"

	Integer, Parameter	:: itime=144, izu_3d=52, izw_3d=52,     &
                                   ix=52, ixu=52, iy=52, iyv=52, izs_3d=8

	Real, Dimension (itime)			:: time
	Real, Dimension (izu_3d)	:: zu_3d, zw_3d
	Real, Dimension (ix)		:: x, E_UTM, Ev_UTM
	Real, Dimension (iy)		:: y, N_UTM, Nu_UTM
	Real, Dimension (ixu)		:: xu, Eu_UTM
	Real, Dimension (iyv)		:: yv, Nv_UTM
	Real, Dimension (iy, ix)	:: lon, lat
	Real, Dimension (iy, ixu)	:: lonu, latu
	Real, Dimension (iyv, ix)	:: lonv, latv
	Real, Dimension (izs_3d)	:: zs_3d
	
	Real, Dimension (ix, iy, izu_3d, itime)	:: e, p, prr, qv, rh,   &
                                                   ta, theta, thetav,   &
                                                   wdir, wspeed
	Real, Dimension (ixu, iy, izu_3d, itime) :: u
	Real, Dimension (ix, iyv, izu_3d, itime) :: v
	Real, Dimension (ix, iy, izw_3d, itime) :: w, rad_sw_in, rad_sw_out, &
                                                   rad_lw_in, rad_lw_out
	Real	:: ahour

	Integer	:: ncid, varid, i, k

	Open (101, file = 'TEST-OUT.dat', form='formatted') 
	Open (102, file = 'TEST2-OUT.dat', form='formatted') 

	Write (101, 10) 'TIME', 'HOUR', 'HEIGHT', 'T', 'RH', 'U', 'V', &
        'WS', 'WD', 'QV', 'THETA', 'THETAV', 'RAD_SW_IN', 'RAD_SW_OUT',&
        'RAD_LW_IN', 'RAD_LW_OUT'
	Write (102, 10) 'TIME', 'HOUR', 'T', 'RH', 'U', 'V', 'WS',     &
        'WD', 'RAD_SW_IN', 'RAD_SW_OUT', 'RAD_LW_IN', 'RAD_LW_OUT' 
 10	Format (30(1x, a10)) 

	call check( nf90_open(FILE_NAME, NF90_NOWRITE, ncid) )

	call check( nf90_inq_varid(ncid, "time", varid) )
	call check( nf90_get_var(ncid, varid, time) )

	call check( nf90_inq_varid(ncid, "zu_3d", varid) )
	call check( nf90_get_var(ncid, varid, zu_3d) )

	call check( nf90_inq_varid(ncid, "zw_3d", varid) )
	call check( nf90_get_var(ncid, varid, zw_3d) )

	call check( nf90_inq_varid(ncid, "x", varid) )
	call check( nf90_get_var(ncid, varid, x) )

	call check( nf90_inq_varid(ncid, "xu", varid) )
	call check( nf90_get_var(ncid, varid, xu) )

	call check( nf90_inq_varid(ncid, "y", varid) )
	call check( nf90_get_var(ncid, varid, y) )

	call check( nf90_inq_varid(ncid, "yv", varid) )
	call check( nf90_get_var(ncid, varid, yv) )

	call check( nf90_inq_varid(ncid, "E_UTM", varid) )
	call check( nf90_get_var(ncid, varid, E_UTM) )

	call check( nf90_inq_varid(ncid, "N_UTM", varid) )
	call check( nf90_get_var(ncid, varid, N_UTM) )

	call check( nf90_inq_varid(ncid, "Eu_UTM", varid) )
	call check( nf90_get_var(ncid, varid, Eu_UTM) )

	call check( nf90_inq_varid(ncid, "Nu_UTM", varid) )
	call check( nf90_get_var(ncid, varid, Nu_UTM) )

	call check( nf90_inq_varid(ncid, "Ev_UTM", varid) )
	call check( nf90_get_var(ncid, varid, Ev_UTM) )

	call check( nf90_inq_varid(ncid, "Nv_UTM", varid) )
	call check( nf90_get_var(ncid, varid, Nv_UTM) )

	call check( nf90_inq_varid(ncid, "lon", varid) )
	call check( nf90_get_var(ncid, varid, lon) )

	call check( nf90_inq_varid(ncid, "lat", varid) )
	call check( nf90_get_var(ncid, varid, lat) )

	call check( nf90_inq_varid(ncid, "lonu", varid) )
	call check( nf90_get_var(ncid, varid, lonu) )

	call check( nf90_inq_varid(ncid, "latu", varid) )
	call check( nf90_get_var(ncid, varid, latu) )

	call check( nf90_inq_varid(ncid, "lonv", varid) )
	call check( nf90_get_var(ncid, varid, lonv) )

	call check( nf90_inq_varid(ncid, "latv", varid) )
	call check( nf90_get_var(ncid, varid, latv) )

	call check( nf90_inq_varid(ncid, "zs_3d", varid) )
	call check( nf90_get_var(ncid, varid, zs_3d) )

	call check( nf90_inq_varid(ncid, "e", varid) )
	call check( nf90_get_var(ncid, varid, e) )

	call check( nf90_inq_varid(ncid, "p", varid) )
	call check( nf90_get_var(ncid, varid, p) )

	call check( nf90_inq_varid(ncid, "prr", varid) )
	call check( nf90_get_var(ncid, varid, prr) )

	call check( nf90_inq_varid(ncid, "qv", varid) )
	call check( nf90_get_var(ncid, varid, qv) )

	call check( nf90_inq_varid(ncid, "rh", varid) )
	call check( nf90_get_var(ncid, varid, rh) )

	call check( nf90_inq_varid(ncid, "ta", varid) )
	call check( nf90_get_var(ncid, varid, ta) )

	call check( nf90_inq_varid(ncid, "theta", varid) )
	call check( nf90_get_var(ncid, varid, theta) )

	call check( nf90_inq_varid(ncid, "thetav", varid) )
	call check( nf90_get_var(ncid, varid, thetav) )

	call check( nf90_inq_varid(ncid, "u", varid) )
	call check( nf90_get_var(ncid, varid, u) )

	call check( nf90_inq_varid(ncid, "v", varid) )
	call check( nf90_get_var(ncid, varid, v) )

	call check( nf90_inq_varid(ncid, "w", varid) )
	call check( nf90_get_var(ncid, varid, w) )

	call check( nf90_inq_varid(ncid, "wdir", varid) )
	call check( nf90_get_var(ncid, varid, wdir) )

	call check( nf90_inq_varid(ncid, "wspeed", varid) )
	call check( nf90_get_var(ncid, varid, wspeed) )

	call check( nf90_inq_varid(ncid, "rad_sw_in", varid) )
	call check( nf90_get_var(ncid, varid, rad_sw_in) )

	call check( nf90_inq_varid(ncid, "rad_sw_out", varid) )
	call check( nf90_get_var(ncid, varid, rad_sw_out) )

	call check( nf90_inq_varid(ncid, "rad_lw_in", varid) )
	call check( nf90_get_var(ncid, varid, rad_lw_in) )

	call check( nf90_inq_varid(ncid, "rad_lw_out", varid) )
	call check( nf90_get_var(ncid, varid, rad_lw_out) )

	call check( nf90_close(ncid) )

	Do i = 1, itime
	Do k = 1, izu_3d

	ahour = time(i)/3600.0

	Write (101, 50) i, ahour, zu_3d(k), ta(26,26,k,i), rh(26,26,k,i),&
        u(26,26,k,i), v(26,26,k,i), wspeed(26,26,k,i), wdir(26,26,k,i),  &
        qv(26,26,k,i)*1000.0, theta(26,26,k,i), thetav(26,26,k,i),       &
        rad_sw_in(26,26,k,i), rad_sw_out(26,26,k,i),                     &
        rad_lw_in(26,26,k,i), rad_lw_out(26,26,k,i)
 50	Format (1x, i10, 30(1x, f10.2))

	Enddo
	Write (101, *)
	Write (101, 10) 'TIME', 'HOUR', 'HEIGHT', 'T', 'RH', 'U', 'V', &
        'WS', 'WD', 'QV', 'THETA', 'THETAV', 'RAD_SW_IN', 'RAD_SW_OUT',&
        'RAD_LW_IN', 'RAD_LW_OUT' 
	Write (102, 50) i, ahour, ta(26,26,2,i), rh(26,26,2,i),        &
        u(26,26,2,i), v(26,26,2,i), wspeed(26,26,2,i), wdir(26,26,2,i),&
        rad_sw_in(26,26,2,i), rad_sw_out(26,26,2,i),                   &
        rad_lw_in(26,26,2,i), rad_lw_out(26,26,2,i)
	Enddo

contains
   	Subroutine check(status)
        integer, intent ( in) :: status
        if(status /= nf90_noerr) then
         print *, trim(nf90_strerror(status))
         stop "Stopped"
        end if
   	End subroutine check

	End program read_palm_3d
