!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! Program for reading the PALM PROFILES OUTPUT NETCDF Data         !!!
!!! This Code must be compiled with the following flags/options:     
!!! gfortran -o read-palm-pr.x -I/usr/include -L/usr/lib64 -lnetcdf -lnetcdff /usr/lib64/libnetcdf.a read-palm-pr.f90
!!! Last Updated (February 24, 2021)                                 !!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	Program read_palm_pr 

	USE netcdf
	implicit none

	Character(len=*), Parameter :: FILE_NAME = "TEST.nc"

	Integer, Parameter	:: itime = 144, ize = 52, izkh = 52, izkm = 52,&
                           izl = 52, izp = 52, izqv = 52, izrh = 52,   &
                           iztheta = 52, izthetav = 52, izu = 52,      &
                           izv = 52, izw = 52, izwqv2 = 52, izwqv = 52,&
                           izwtheta2 = 52, izwtheta = 52,              &
                           izrad_lw_in = 52, izrad_lw_out = 52,        &
                           izrad_sw_in = 52, izrad_sw_out = 52

    	Real, Dimension(itime)              :: time
    	Real, Dimension(ize)                :: ze  
    	Real, Dimension(ize,itime)          :: e
    	Real, Dimension(izp,itime)          :: p
    	Real, Dimension(izqv,itime)         :: qv
    	Real, Dimension(izrh,itime)         :: rh
    	Real, Dimension(iztheta,itime)      :: theta
    	Real, Dimension(izthetav,itime)     :: thetav        
    	Real, Dimension(izu,itime)          :: u        
    	Real, Dimension(izv,itime)          :: v
    	Real, Dimension(izw,itime)          :: w
    	Real, Dimension(izwqv,itime)        :: wqv
    	Real, Dimension(izwtheta,itime)     :: wtheta
    	Real, Dimension(izrad_lw_in,itime)  :: rad_lw_in
    	Real, Dimension(izrad_lw_out,itime) :: rad_lw_out
    	Real, Dimension(izrad_sw_in,itime)  :: rad_sw_in
    	Real, Dimension(izrad_sw_out,itime) :: rad_sw_out
        
	Real	:: ahour

	Integer	:: ncid, varid, i, k

	Open (101, file = 'PR-PROFILE-OUT.dat', form='formatted') 
	Open (102, file = 'PR-TIME-SERIES.dat', form='formatted') 

	Write (101, 10) 'TIME', 'HOUR', 'HEIGHT', 'TKE', 'P', 'QV', 'RH',  &
                    'THETA', 'THETAV', 'U', 'V', 'W', 'LHF', 'SHF',    &
                    'LWIN', 'LWOUT', 'SWIN', 'SWOUT'
	
	Write (102, 10) 'TIME', 'HOUR', 'HEIGHT', 'TKE', 'P', 'QV', 'RH',  &
                    'THETA', 'THETAV', 'U', 'V', 'W', 'LHF', 'SHF',    &
                    'LWIN', 'LWOUT', 'SWIN', 'SWOUT'
 10	Format (30(1x, a10)) 

	call check( nf90_open(FILE_NAME, NF90_NOWRITE, ncid) )

	call check( nf90_inq_varid(ncid, "time", varid) )
	call check( nf90_get_var(ncid, varid, time) )

	call check( nf90_inq_varid(ncid, "ze", varid) )
	call check( nf90_get_var(ncid, varid, ze) )

	call check( nf90_inq_varid(ncid, "e", varid) )
	call check( nf90_get_var(ncid, varid, e) )

	call check( nf90_inq_varid(ncid, "p", varid) )
	call check( nf90_get_var(ncid, varid, p) )

	call check( nf90_inq_varid(ncid, "qv", varid) )
	call check( nf90_get_var(ncid, varid, qv) )

	call check( nf90_inq_varid(ncid, "rh", varid) )
	call check( nf90_get_var(ncid, varid, rh) )

	call check( nf90_inq_varid(ncid, "theta", varid) )
	call check( nf90_get_var(ncid, varid, theta) )

	call check( nf90_inq_varid(ncid, "thetav", varid) )
	call check( nf90_get_var(ncid, varid, thetav) )

	call check( nf90_inq_varid(ncid, "u", varid) )
	call check( nf90_get_var(ncid, varid, u) )

	call check( nf90_inq_varid(ncid, "v", varid) )
	call check( nf90_get_var(ncid, varid, v) )

	call check( nf90_inq_varid(ncid, "w", varid) )
	call check( nf90_get_var(ncid, varid, w) )

	call check( nf90_inq_varid(ncid, "wqv", varid) )
	call check( nf90_get_var(ncid, varid, wqv) )

	call check( nf90_inq_varid(ncid, "wtheta", varid) )
	call check( nf90_get_var(ncid, varid, wtheta) )

	call check( nf90_inq_varid(ncid, "rad_lw_in", varid) )
	call check( nf90_get_var(ncid, varid, rad_lw_in) )

	call check( nf90_inq_varid(ncid, "rad_lw_out", varid) )
	call check( nf90_get_var(ncid, varid, rad_lw_out) )

	call check( nf90_inq_varid(ncid, "rad_sw_in", varid) )
	call check( nf90_get_var(ncid, varid, rad_sw_in) )

	call check( nf90_inq_varid(ncid, "rad_sw_out", varid) )
	call check( nf90_get_var(ncid, varid, rad_sw_out) )

	call check( nf90_close(ncid) )

	Do i = 1, itime
	Do k = 1, ize

	ahour = time(i)/3600.0

	Write (101, 50) i, ahour, ze(k), e(k,i), p(k,i), qv(k,i),      &
                        rh(k,i), theta(k,i), thetav(k,i), u(k,i),      &
                        v(k,i), w(k,i), wqv(k,i), wtheta(k,i),         &
                        rad_lw_in(k,i), rad_lw_out(k,i),               &
                        rad_sw_in(k,i), rad_sw_out(k,i)
 50	Format (1x, i10, 30(1x, f10.2))

	Enddo
	Write (101, *)
	Write (102, 50) i, ahour, ze(2), e(2,i), p(2,i), qv(2,i),      &
                        rh(2,i), theta(2,i), thetav(2,i), u(2,i),      &
                        v(2,i), w(2,i), wqv(2,i), wtheta(2,i),         &
                        rad_lw_in(2,i), rad_lw_out(2,i),               &
                        rad_sw_in(2,i), rad_sw_out(2,i)
	Enddo

contains
   	Subroutine check(status)
        integer, intent ( in) :: status
        if(status /= nf90_noerr) then
         print *, trim(nf90_strerror(status))
         stop "Stopped"
        end if
   	End subroutine check

	End program read_palm_pr
