!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! Program for reading the PALM TIME-SERIES OUTPUT NETCDF Data      !!!
!!! This Code must be compiled with the following flags/options:     
!!! gfortran -o read-ts.x -I/usr/include -L/usr/lib64 -lnetcdf -lnetcdff /usr/lib64/libnetcdf.a read-ts.f90
!!! Last Updated (February 16, 2021)                                 !!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	Program read_time_series

	USE netcdf
	implicit none

	Character(len=*), Parameter :: FILE_NAME = "les2_05Dec2017_ts.000.nc"

	Integer, Parameter	:: itime=9262

	Real, Dimension(itime)	:: time, E, dt, umax, vmax, wmax, ghf,  & 
                                   rad_net, rad_lw_in, rad_lw_out,      &
                                   rad_sw_in, rad_sw_out, wtheta 
	Real	:: ahour
	Integer :: i

	Integer	:: ncid, varid

	Open (101, file = 'LES2-05DEC2017-TIME-SERIES.dat', form='formatted') 

	Write (101, 10) 'TIME', 'HOUR', 'TKE', 'DT', 'UMAX', 'VMAX',    &
                        'WMAX', 'GHF', 'RAD_NET', 'RAD_LW_IN',          &
                        'RAD_LW_OUT', 'RAD_SW_IN', 'RAD_SW_OUT', 'SHF'
 10	Format (25(1x, a10))

	call check( nf90_open(FILE_NAME, NF90_NOWRITE, ncid) )

	call check( nf90_inq_varid(ncid, "time", varid) )
	call check( nf90_get_var(ncid, varid, time) )

	call check( nf90_inq_varid(ncid, "E", varid) )
	call check( nf90_get_var(ncid, varid, E) )

	call check( nf90_inq_varid(ncid, "dt", varid) )
	call check( nf90_get_var(ncid, varid, dt) )

	call check( nf90_inq_varid(ncid, "umax", varid) )
	call check( nf90_get_var(ncid, varid, umax) )

	call check( nf90_inq_varid(ncid, "vmax", varid) )
	call check( nf90_get_var(ncid, varid, vmax) )

	call check( nf90_inq_varid(ncid, "ghf", varid) )
	call check( nf90_get_var(ncid, varid, ghf) )

	call check( nf90_inq_varid(ncid, "rad_net", varid) )
	call check( nf90_get_var(ncid, varid, rad_net) )

	call check( nf90_inq_varid(ncid, "rad_lw_in", varid) )
	call check( nf90_get_var(ncid, varid, rad_lw_in) )

	call check( nf90_inq_varid(ncid, "rad_lw_out", varid) )
	call check( nf90_get_var(ncid, varid, rad_lw_out) )

	call check( nf90_inq_varid(ncid, "rad_sw_in", varid) )
	call check( nf90_get_var(ncid, varid, rad_sw_in) )

	call check( nf90_inq_varid(ncid, "rad_sw_out", varid) )
	call check( nf90_get_var(ncid, varid, rad_sw_out) )
	
	call check( nf90_inq_varid(ncid, "wtheta", varid) )
	call check( nf90_get_var(ncid, varid, wtheta) )

	call check( nf90_close(ncid) )

	Do i = 1, itime

	ahour = time(i)/3600.0

	Write (101, 15) time(i), ahour, E(i), dt(i), umax(i), vmax(i),  &
                        wmax(i), ghf(i), rad_net(i), rad_lw_in(i),      &
                        rad_lw_out(i), rad_sw_in(i), rad_sw_out(i), &
                        wtheta(i)
 15	Format (1x, f10.1, 1x, f10.4, 25(1x, f10.2))

	Enddo

contains
   	Subroutine check(status)
        integer, intent ( in) :: status
        if(status /= nf90_noerr) then
         print *, trim(nf90_strerror(status))
         stop "Stopped"
        end if
   	End subroutine check

	End program read_time_series
